@@ -4,6 +4,7 @@
 import subprocess
 from tempfile import NamedTemporaryFile
 from textwrap import dedent
+from unittest.mock import Mock
 
 from .._confirm import Confirmation
 from .._engine import DeleteMergedBranches
@@ -32,12 +33,14 @@ def run_script(content, cwd):
 
 def create_git(work_dir: str) -> Git:
     messenger = Messenger(colorize=False)
-    return Git(messenger=messenger, pretend=False, verbose=True,
-               work_dir=work_dir)
+    return Git(messenger=messenger, pretend=False, verbose=True, work_dir=work_dir)
 
 
 def create_dmb(git: Git, effort_level: int) -> DeleteMergedBranches:
     messenger = Messenger(colorize=False)
     confirmation = Confirmation(messenger=messenger, ask=False)
-    return DeleteMergedBranches(git, messenger=messenger, confirmation=confirmation,
+    return DeleteMergedBranches(git,
+                                messenger=messenger,
+                                confirmation=confirmation,
+                                selector=Mock(),
                                 effort_level=effort_level)
